==> IceWarpServer-10.0.6_RHEL5.tar.gz: html/webmail/client/inc/frm_chat.js <==
_me = frm_chat.prototype;
function frm_chat(){};

_me.__constructor = function() {
	var me = this;

    this._defaultSize(-1,-1,500,350);

	this.__tabIndex = {};

	this._draw('frm_chat_top','header');

	this.history._onclick = function(){
		var sJID = me._getName(me.tabs._value());
		gui._create('frm_im_history', 'frm_im_history', '', '', me.__im.__xmpp, sJID);
	};
	
	this.vcard._onclick = function(){
		var sJID = me._getName(me.tabs._value());
		gui._create('frm_im_vcard', 'frm_im_vcard', '', '', sJID, [me.__im.__xmpp,'vcard']);
	};
	this.clear._onclick = function(){
		var	tab = me.tabs._value(),
			sJID = me._getName(tab);
			
		dataSet.add('xmpp',['roster',sJID,'history'],[]);
		me.tabs[tab].chat._clear();
		me.tabs[tab].text._focus();
		
		//clear history on server
	};
	
	this.jid.__value = function (v){
		if (v.indexOf('~')===0){
			var aTmp = dataSet.get('xmpp', ['roster']),
				g = v.replace(/^\~/g,'');
				
			v = '';
			for(var i in aTmp)
				if (aTmp[i].group == g)
					v += v?'; '+i:i;

			v = v || g;
		}
		else{
			var sStatus = dataSet.get('xmpp',['roster',v,'show']);
			if (sStatus)
				v += ' ('+ (getLang('STATUS::'+sStatus.toUpperCase()) || getLang('STATUS::OFFLINE')) +')';
		}

		this._value(v);
	};
	
	//Tab part
	this._create('tabs','obj_tabs');

	//Send button
	this._create('btn_send', 'obj_button', 'footer');
	this.btn_send._value('COMPOSE::SEND');
	this.btn_send._onclick = function(){
		var sTo, aTo,
			v = me.tabs._value(),
			b = me.tabs[v].text._value();

		if (b.length){
			for (var i in me.__tabIndex)
				if (me.__tabIndex[i] == v){
					sTo = i;
					break;
				}

			if (sTo.indexOf('~')===0){
				var aTmp = dataSet.get('xmpp', ['roster']),
					g = sTo.replace(/^\~/g,''),
					aTo = [];

				for(var i in aTmp)
					if (aTmp[i].group == g)
						aTo.push(i);
			}

			if (me.__im._send(sTo, b, aTo))
				if (me && me.tabs && me.tabs[v])
					me.tabs[v].text._value('');
		}

		if (me && me.tabs && me.tabs[v])
			me.tabs[v].text._focus(true);
	};
};


// user is composing...
_me._compose = function(sJID){
	var tab,
		tmp = this.__tabIndex[sJID.split('/').shift()];
	
	if (sJID && tmp && (tab = this.tabs[tmp]))
		addcss(tab.__eLink, 'im_compose');
};
_me._wait = function (sJID){
	var tab,
		tmp = this.__tabIndex[sJID.split('/').shift()];
		
	if (sJID && tmp && (tab = this.tabs[tmp]))
		removecss(tab.__eLink, 'im_compose');
};


_me._getName = function(sTab){
	for (var i in this.__tabIndex)
		if (this.__tabIndex[i] == sTab)
			return i;

	return false;
};

// return focus back after move/undock
_me._onresize_end = function (){
	var tab = this.tabs._value();
	if (tab && (tab = this.tabs[tab]))
		tab.text._focus(true);

	return true;
};

_me._onundock = function (){
	var tab = this.tabs._value();
	if (tab && (tab = this.tabs[tab])){
		//set focus
		tab.text._focus(true);
		//scroll chat
		tab.chat._scroll();
	}

	return true;
};


_me._tab = function(sFrom, bActive){

	var sName, tab = null, me = this;
	if (sFrom.indexOf('~')===0){
        sName = sFrom;
        sName = sName.replace(/^\~/g,'');

		if (sName == '*')
		    sName = '['+getLang('IM::UNSORTED')+']';
		else
		if (sName == '~')
		    sName = '['+getLang('IM::NOT_LISTED')+']';
	}
	else
		sName = dataSet.get('xmpp', ['roster', sFrom, 'name']) || sFrom;

	//New Tab
	if (!this.__tabIndex[sFrom] || !this.tabs[this.__tabIndex[sFrom]]){

		//auto active first tab
		if (!count(this.__tabIndex))
			bActive = true;

		tab = this.tabs._create('tab','obj_tab_close');
		tab._value(sName,true);
		tab._onclose = function(){
			if (this._parent._value() == '')
				me._destruct();
		};

		tab._oncontext = function(e){

			if (!me.__im) return;

			var id,
				bLogin = !me.__im._is_active();
				
			for(var i in me.__tabIndex)
				if (me.__tabIndex[i] == this._name){
				    id = i;
				    break;
				}
				
			if (!id || id.indexOf('~')==0) return;

			var aMenu = [
				{'title':'IM::VCARD','arg':{'method':'vcard',id:id},disabled:bLogin},
				{'title':'IM::SEND_MAIL','arg':{'method':'send',id:id}},
				{'title':'IM::SEND_FILE','arg':{'method':'file',id:id},disabled:bLogin},
				{'title':'-'},
				{'title':'IM::SUBSCRIPTION',nodes:[
					{'title':'IM::SEND','arg':{method:'sub_add',id:id}},
					{'title':'IM::REQUEST','arg':{method:'sub_get',id:id}},
					{'title':'IM::REMOVE','arg':{method:'sub_remove',id:id}}
				],disabled:bLogin}
				];

			var cmenu = gui._create("cmenu","obj_context",'','',this);
				cmenu._fill(aMenu);
				cmenu._place(e.clientX,e.clientY);
				cmenu._onclick = function(e,elm,id,arg){
					me.__im._oncontext(e,elm,id,arg);
				};
		};

		tab._onactive = function(){
			me._title(this._value(), true);
			this.text._focus();

			removecss(this.__eLink, 'im_event');

			for(var i in me.__tabIndex)
				if (me.__tabIndex[i] == this._name){
					me.jid.__value(i);
					break;
				}

			//deactivate button for group message
			var b = (i.indexOf('~')===0);
			me.history._disabled(b);
			me.vcard._disabled(b);
			me.clear._disabled(b);

			//scroll chat if exists
			if (this.chat)
				this.chat._scroll();
		};

		tab.__update = function(sDName,sDPath){
			if (sDPath && arrayCompare(sDPath,this._listenerPath)){
				tab.chat._status(sDPath[1],dataSet.get(sDName,this._listenerPath));

				if (this._isActive)
					me.jid.__value(sDPath[1]);
			}
		};

		if (bActive){
			this._title(tab._value(), true);
			this.jid.__value(sFrom);
			
            //deactivate button for group message
			var b = (sFrom.indexOf('~')===0);
			this.history._disabled(b);
			this.vcard._disabled(b);
			this.clear._disabled(b);
		}

		//remove tab from tabindex
		tab.__onDestruct = function (){
			for(var i in me.__tabIndex)
				if (me.__tabIndex[i] == this._name){
					delete me.__tabIndex[i];
					break;
				}
		};
		tab._add_destructor('__onDestruct');


		tab._draw('frm_chat_form','main',{image:'./server/download.php?class=avatar&fullpath='+ sFrom});

/*
		//RESIZE
		var eScr = document.getElementById(tab._pathName+'scroll');
		    eScr.onmousedown = function(e){
				var e = e || window.event;

				var eCon = document.getElementById(tab._pathName+'cont');
				var p = getSize(this).y;
				var y = e.clientY - p;

				tab._mousemove_bar = function (e){
					iPos = e.clientX - spos;
				};

				gui._obeyEvent('mousemove',[me._mousemove_bar]);
			};
*/

		//Send (Ctrl + Enter)
		tab.text.__eIN.onkeydown = function(e){

			var e = e || window.event;

			//CLOSE KEY
			if (e.keyCode == 27){
				//kill esc key becuase of closing obj_popup
				e.cancelBubble = true;
				try{e.preventDefault();}catch(r){}
				try{e.stopPropagation();}catch(r){}

				tab._close();
				return false;
			}
			else
			//SEND KEY
			if (e.keyCode == 13){
				if ((GWOthers.getItem('IM','enter_send')>0 && !e.shiftKey) || e.ctrlKey){
					e.cancelBubble = true;
					try{e.preventDefault();}catch(r){}
					try{e.stopPropagation();}catch(r){}

					me.btn_send._onclick();
					return false;
				}
			}
		};

        if (bActive)
			tab.text._focus();
			
        this.__tabIndex[sFrom] = tab._name;

		//Load history
		var h = dataSet.get('xmpp',['roster',sFrom,'history']);
		if (h)
		    tab.chat._fill(h);

		tab._listen('xmpp',['roster', sFrom, 'show'],true);
	}
	else
		tab = this.tabs[this.__tabIndex[sFrom]];

	if (bActive)
		tab._active();

	return tab;
};

_me.__history = function(aData, sFrom, sTo, sBody, date){
/*
 [0]
.. [CHAT]
.... [0]
...... [TO]
........ [0]
.......... [BODY]
............ [0]
.............. [VALUE](string) = sdfsdf
.......... [ATTRIBUTES]
............ [SECS](string) = 44164
............ [JID](string) = admin@merakdemo.com
........ [1]
.......... [BODY]
............ [0]
.............. [VALUE](string) = sdfsdf
.......... [ATTRIBUTES]
............ [SECS](string) = 44177
............ [JID](string) = admin@merakdemo.com
...... [FROM]
........ [0]
.......... [BODY]
............ [0]
.............. [VALUE](string) = sdsdf
.......... [ATTRIBUTES]
............ [SECS](string) = 44173
............ [JID](string) = user@merakdemo.com
........ [1]
.......... [BODY]
............ [0]
.............. [VALUE](string) = sdfsdf
.......... [ATTRIBUTES]
............ [SECS](string) = 44181
............ [JID](string) = user@merakdemo.com
...... [SET]
........ [0]
.......... [FIRST]
............ [0]
.............. [VALUE](string) = 44164
.......... [LAST]
............ [0]
.............. [VALUE](string) = 44181
.......... [ATTRIBUTES]
............ [XMLNS](string) = http://jabber.org/protocol/rsm
...... [ATTRIBUTES]
........ [XMLNS](string) = urn:xmpp:archive
........ [WITH](string) = user@merakdemo.com
........ [START](string) = 2009-06-04T00:00:00Z
........ [VERSION](string) = 1
.. [ATTRIBUTES]
.... [ID](string) = 6705748755706382001244189821445
.... [TO](string) = admin@merakdemo.com/Webmail
.... [TYPE](string) = result
 [1](string) = user@merakdemo.com
*/

	var tmd, arr = [];
	if (aData.CHAT[0].TO){
		var sName = dataSet.get('main',['fullname']) || dataSet.get('main',['user']);
		for (var i in aData.CHAT[0].TO)
			if (aData.CHAT[0].TO[i].BODY && aData.CHAT[0].TO[i].BODY[0] && aData.CHAT[0].TO[i].BODY[0].VALUE){
				tmp = new Date();
				tmp.setVersit(aData.CHAT[0].ATTRIBUTES.START,aData.CHAT[0].TO[i].ATTRIBUTES.SECS);
				arr.push({from: sName, body:aData.CHAT[0].TO[i].BODY[0].VALUE, reply:true, date:tmp});
			}
	}

	if (aData.CHAT[0].FROM)
		for (var i in aData.CHAT[0].FROM)
			if (aData.CHAT[0].FROM[i].BODY && aData.CHAT[0].FROM[i].BODY[0] && aData.CHAT[0].FROM[i].BODY[0].VALUE){
				tmp = new Date();
				tmp.setVersit(aData.CHAT[0].ATTRIBUTES.START,aData.CHAT[0].FROM[i].ATTRIBUTES.SECS);
				arr.push ({from: aData.CHAT[0].FROM[i].ATTRIBUTES.JID, body:aData.CHAT[0].FROM[i].BODY[0].VALUE, reply:false, date:tmp});
			}

	if (arr.length>0)
		arr.sort(function arrsrt (a,b){
			return a.date-b.date;
		});

	dataSet.add('xmpp',['roster',sFrom,'history'], arr);

	this._chat(sFrom, sTo, sBody, date, true);
};

_me._chat = function(sFrom, sTo, sBody, date, bHistory){
	var sFrom = sFrom.split('/').shift();
        date = date || new Date();

	if (sFrom.indexOf('~')!=0 && typeof dataSet.get('xmpp',['roster',sFrom,'history']) == 'undefined'){
        var oDate = new Date();
        	oDate.setHours(-48,0,0);

		this.__im.__xmpp.history_get({'from':sFrom,'max':20,'before':''},[this,'__history',[sFrom, sTo, sBody, date]]);
		return;
	}

	if (sBody){
		var tab = this._tab(sFrom);

		removecss(tab.__eLink, 'im_compose');
		
		if (!tab._isActive)
			addcss(tab.__eLink, 'im_event');

		var h = dataSet.get('xmpp',['roster',sFrom,'history']);
		if (!bHistory || (bHistory && h.length == 0)){
			if (sTo)
				tab.chat._add (sTo, sBody, true, date);
			else
				tab.chat._add (sFrom, sBody, false, date);
				
			//Save history
			if (!h)
			    h = [];
			else
			if (h.length>49)
			    h = h.slice(h.length-49)

			h.push({from: sTo || sFrom,body:sBody,reply:sTo?true:false,date:date});
			dataSet.add('xmpp',['roster',sFrom,'history'],h,true);
		}
	}
	else{
		if (this._docked)
			this._undock();

		this._tab(sFrom,true);
	}
};

==> IceWarpServer-10.1.4_RHEL5.tar.gz: html/webmail/client/inc/frm_chat.js <==
_me = frm_chat.prototype;
function frm_chat(){};

_me.__constructor = function() {
	var me = this;

    this._defaultSize(-1,-1,500,350);

	this.__tabIndex = {};
	this.__lastHistory = {};

	this._draw('frm_chat_top','header');

	this.history._onclick = function(){
		var sJID = me._getName(me.tabs._value());
		gui._create('frm_im_history', 'frm_im_history', '', '', me.__im.__xmpp, sJID);
	};
	
	this.vcard._onclick = function(){
		var sJID = me._getName(me.tabs._value());
		gui._create('frm_im_vcard', 'frm_im_vcard', '', '', sJID, [me.__im.__xmpp,'vcard']);
	};
	this.clear._onclick = function(){
		var	tab = me.tabs._value(),
			sJID = me._getName(tab);
			
		dataSet.add('xmpp',['roster',sJID,'history'],[]);
		me.tabs[tab].chat._clear();
		me.tabs[tab].text._focus();
		
		//clear history on server
	};
	
	this.jid.__value = function (v){
		if (v.indexOf('~')===0){
			var aTmp = dataSet.get('xmpp', ['roster']),
				g = v.replace(/^\~/g,'');
				
			v = '';
			for(var i in aTmp)
				if (aTmp[i].group == g)
					v += v?'; '+i:i;

			v = v || g;
		}
		else{
			var sStatus = dataSet.get('xmpp',['roster',v,'show']);
			if (sStatus)
				v += ' ('+ (getLang('STATUS::'+sStatus.toUpperCase(),'',2) || getLang('STATUS::OFFLINE')) +')';
		}

		this._value(v);
	};
	
	//Tab part
	this._create('tabs','obj_tabs');

	//Send button
	this._create('btn_send', 'obj_button', 'footer');
	this.btn_send._value('COMPOSE::SEND');
	this.btn_send._onclick = function(){
		var sTo, aTo,
			v = me.tabs._value(),
			b = me.tabs[v].text._value();

		if (b.length){
			for (var i in me.__tabIndex)
				if (me.__tabIndex[i] == v){
					sTo = i;
					break;
				}

			if (sTo.indexOf('~')===0){
				var g = sTo.replace(/^\~/g,''),
					aTo = [];

				if (g.indexOf(';')>-1)
					aTo = g.split('; ');
				else{
					var aTmp = dataSet.get('xmpp', ['roster']);
					for(var i in aTmp)
						if (aTmp[i].group == g)
							aTo.push(i);
				}
			}

			if (me.__im._send(sTo, b, aTo))
				if (me && me.tabs && me.tabs[v])
					me.tabs[v].text._value('');
		}

		if (me && me.tabs && me.tabs[v])
			me.tabs[v].text._focus(true);
	};
};


// user is composing...
_me._compose = function(sJID){
	var tab,
		tmp = this.__tabIndex[sJID.split('/').shift()];
	
	if (sJID && tmp && (tab = this.tabs[tmp]))
		addcss(tab.__eLink, 'im_compose',true);
};
_me._wait = function (sJID){
	var tab,
		tmp = this.__tabIndex[sJID.split('/').shift()];
		
	if (sJID && tmp && (tab = this.tabs[tmp]))
		removecss(tab.__eLink, 'im_compose',true);
};


_me._getName = function(sTab){
	for (var i in this.__tabIndex)
		if (this.__tabIndex[i] == sTab)
			return i;

	return false;
};

// return focus back after move/undock
_me._onresize_end = function (){
	var tab = this.tabs._value();
	if (tab && (tab = this.tabs[tab]))
		tab.text._focus(true);

	return true;
};

_me._onundock = function (){
	var tab = this.tabs._value();
	if (tab && (tab = this.tabs[tab])){
		//set focus
		tab.text._focus(true);
		//scroll chat
		tab.chat._scroll();
	}

	return true;
};


_me._tab = function(sFrom, bActive){

	var sName, tab = null, me = this;
	if (sFrom.indexOf('~')===0){
        sName = sFrom;
        sName = sName.replace(/^\~/g,'');

		if (sName == '*')
		    sName = '['+getLang('IM::UNSORTED')+']';
		else
		if (sName == '~')
		    sName = '['+getLang('IM::NOT_LISTED')+']';
		else
		if (sFrom.indexOf(';')>-1)
			sName = '['+getLang('IM::SELECTED')+']';
	}
	else
		sName = dataSet.get('xmpp', ['roster', sFrom, 'name']) || sFrom;

	//New Tab
	if (!this.__tabIndex[sFrom] || !this.tabs[this.__tabIndex[sFrom]]){

		//auto active first tab
		if (!count(this.__tabIndex))
			bActive = true;

		tab = this.tabs._create('tab','obj_tab_close');
		tab._value(sName,true);
		tab._onclose = function(){
			if (this._parent._value() == '')
				me._destruct();
		};

		tab._oncontext = function(e){

			if (!me.__im) return;

			var id,
				bLogin = !me.__im._is_active();
				
			for(var i in me.__tabIndex)
				if (me.__tabIndex[i] == this._name){
				    id = i;
				    break;
				}
				
			if (!id || id.indexOf('~')==0) return;

			var aMenu = [
				{'title':'IM::VCARD','arg':{'method':'vcard',id:id},disabled:bLogin},
				{'title':'IM::SEND_MAIL','arg':{'method':'send',id:id}},
				{'title':'IM::SEND_FILE','arg':{'method':'file',id:id},disabled:bLogin},
				{'title':'-'},
				{'title':'IM::SUBSCRIPTION',nodes:[
					{'title':'IM::SEND','arg':{method:'sub_add',id:id}},
					{'title':'IM::REQUEST','arg':{method:'sub_get',id:id}},
					{'title':'IM::REMOVE','arg':{method:'sub_remove',id:id}}
				],disabled:bLogin}
				];

			var cmenu = gui._create("cmenu","obj_context",'','',this);
				cmenu._fill(aMenu);
				cmenu._place(e.clientX,e.clientY);
				cmenu._onclick = function(e,elm,id,arg){
					me.__im._oncontext(e,elm,id,arg);
				};
		};

		tab._onactive = function(){
			me._title(this._value(), true);
			removecss(this.__eLink, 'im_event',true);

			this.text._focus();

			for(var i in me.__tabIndex)
				if (me.__tabIndex[i] == this._name){
					me.jid.__value(i);

					//COOKIE - odebrat
					if (Cookie.get(['im','queue',sFrom]) == 1){
						Cookie.set(['im','queue',sFrom]);
						dataSet.remove('xmpp', ['roster', sFrom, 'action']);

						if (gui.frm_main.lister)
							gui.frm_main.lister._checkIM();
					}

					break;
				}

			//deactivate button for group message
			var b = (i.indexOf('~')===0);
			me.history._disabled(b);
			me.vcard._disabled(b);
			me.clear._disabled(b);

			//scroll chat if exists
			if (this.chat)
				this.chat._scroll();
		};

		tab.__update = function(sDName,sDPath){
			if (sDPath && arrayCompare(sDPath,this._listenerPath)){
				tab.chat._status(sDPath[1],dataSet.get(sDName,this._listenerPath));

				if (this._isActive)
					me.jid.__value(sDPath[1]);
			}
		};

		if (bActive){
			this._title(tab._value(), true);
			this.jid.__value(sFrom);
			
            //deactivate button for group message
			var b = (sFrom.indexOf('~')===0);
			this.history._disabled(b);
			this.vcard._disabled(b);
			this.clear._disabled(b);
		}

		//remove tab from tabindex
		tab.__onDestruct = function (){
			for(var i in me.__tabIndex)
				if (me.__tabIndex[i] == this._name){
					delete me.__tabIndex[i];
					break;
				}
		};
		tab._add_destructor('__onDestruct');

		tab._draw('frm_chat_form','main',{image:'./server/download.php?class=avatar&fullpath='+ sFrom});

/*
		//RESIZE
		var eScr = document.getElementById(tab._pathName+'scroll');
		    eScr.onmousedown = function(e){
				var e = e || window.event;

				var eCon = document.getElementById(tab._pathName+'cont');
				var p = getSize(this).y;
				var y = e.clientY - p;

				tab._mousemove_bar = function (e){
					iPos = e.clientX - spos;
				};

				gui._obeyEvent('mousemove',[me._mousemove_bar]);
			};
*/

		//Send (Ctrl + Enter)
		tab.text.__eIN.onkeydown = function(e){

			var e = e || window.event;

			//CLOSE KEY
			if (e.keyCode == 27){
				//kill esc key becuase of closing obj_popup
				e.cancelBubble = true;
				try{e.preventDefault();}catch(r){}
				try{e.stopPropagation();}catch(r){}

				tab._close();
				return false;
			}
			else
			//SEND KEY
			if (e.keyCode == 13){
				if ((GWOthers.getItem('IM','enter_send')>0 && !e.shiftKey) || e.ctrlKey){
					e.cancelBubble = true;
					try{e.preventDefault();}catch(r){}
					try{e.stopPropagation();}catch(r){}

					me.btn_send._onclick();
					return false;
				}
			}
		};

        this.__tabIndex[sFrom] = tab._name;

		//Load history
		var h = dataSet.get('xmpp',['roster',sFrom,'history']);
		if (h)
		    tab.chat._fill(h);

		tab._listen('xmpp',['roster', sFrom, 'show'],true);
	}
	else
		tab = this.tabs[this.__tabIndex[sFrom]];

	if (bActive){
		tab._active();
		tab.text._focus();
	}

	return tab;
};

_me.__history = function(aData, sFrom, sTo, sBody, date){
	//history serialization
	if (this.__lastHistory[sFrom]!==date)
		return;
/*
 [0]
.. [CHAT]
.... [0]
...... [TO]
........ [0]
.......... [BODY]
............ [0]
.............. [VALUE](string) = sdfsdf
.......... [ATTRIBUTES]
............ [SECS](string) = 44164
............ [JID](string) = admin@merakdemo.com
........ [1]
.......... [BODY]
............ [0]
.............. [VALUE](string) = sdfsdf
.......... [ATTRIBUTES]
............ [SECS](string) = 44177
............ [JID](string) = admin@merakdemo.com
...... [FROM]
........ [0]
.......... [BODY]
............ [0]
.............. [VALUE](string) = sdsdf
.......... [ATTRIBUTES]
............ [SECS](string) = 44173
............ [JID](string) = user@merakdemo.com
........ [1]
.......... [BODY]
............ [0]
.............. [VALUE](string) = sdfsdf
.......... [ATTRIBUTES]
............ [SECS](string) = 44181
............ [JID](string) = user@merakdemo.com
...... [SET]
........ [0]
.......... [FIRST]
............ [0]
.............. [VALUE](string) = 44164
.......... [LAST]
............ [0]
.............. [VALUE](string) = 44181
.......... [ATTRIBUTES]
............ [XMLNS](string) = http://jabber.org/protocol/rsm
...... [ATTRIBUTES]
........ [XMLNS](string) = urn:xmpp:archive
........ [WITH](string) = user@merakdemo.com
........ [START](string) = 2009-06-04T00:00:00Z
........ [VERSION](string) = 1
.. [ATTRIBUTES]
.... [ID](string) = 6705748755706382001244189821445
.... [TO](string) = admin@merakdemo.com/Webmail
.... [TYPE](string) = result
 [1](string) = user@merakdemo.com
*/

	var tmd, arr = [];
	if (aData.CHAT[0].TO){
		var sName = dataSet.get('main',['fullname']) || dataSet.get('main',['user']);
		for (var i in aData.CHAT[0].TO)
			if (aData.CHAT[0].TO[i].BODY && aData.CHAT[0].TO[i].BODY[0] && aData.CHAT[0].TO[i].BODY[0].VALUE){
				tmp = new Date();
				tmp.setVersit(aData.CHAT[0].ATTRIBUTES.START,aData.CHAT[0].TO[i].ATTRIBUTES.SECS);
				arr.push({from: sName, body:aData.CHAT[0].TO[i].BODY[0].VALUE, reply:true, date:tmp});
			}
	}

	if (aData.CHAT[0].FROM)
		for (var i in aData.CHAT[0].FROM)
			if (aData.CHAT[0].FROM[i].BODY && aData.CHAT[0].FROM[i].BODY[0] && aData.CHAT[0].FROM[i].BODY[0].VALUE){
				tmp = new Date();
				tmp.setVersit(aData.CHAT[0].ATTRIBUTES.START,aData.CHAT[0].FROM[i].ATTRIBUTES.SECS);
				arr.push ({from: aData.CHAT[0].FROM[i].ATTRIBUTES.JID, body:aData.CHAT[0].FROM[i].BODY[0].VALUE, reply:false, date:tmp});
			}

	if (arr.length>0)
		arr.sort(function arrsrt (a,b){
			return a.date-b.date;
		});

	dataSet.add('xmpp',['roster',sFrom,'history'], arr);

	this._chat(sFrom, sTo, sBody, date, true);
};

_me._chat = function(sFrom, sTo, sBody, date, bHistory){
	var sFrom = sFrom.split('/').shift();
        date = date || new Date();

	if (sPrimaryAccountIMHISTORY && sFrom.indexOf('~')!=0 && typeof dataSet.get('xmpp',['roster',sFrom,'history']) == 'undefined'){
        this.__lastHistory[sFrom] = date;
		this.__im.__xmpp.history_get({'from':sFrom,'max':20,'before':''},[this,'__history',[sFrom, sTo, sBody, date]]);
		return;
	}

	if (sBody){
		var tab = this._tab(sFrom);

		removecss(tab.__eLink, 'im_compose',true);
		
		if (!tab._isActive){
			addcss(tab.__eLink, 'im_event',true);

			//COOKIE - zapsat
			dataSet.add('xmpp', ['roster', sFrom, 'action'], 'msg');

			Cookie.set(['im','queue',sFrom], 1);
			if (gui.frm_main.lister)
				gui.frm_main.lister._checkIM();
		}
		else
			dataSet.remove('xmpp', ['roster', sFrom, 'action']);

		var h = dataSet.get('xmpp',['roster',sFrom,'history']);
		if (!bHistory || (bHistory && h.length == 0)){
			if (sTo)
				tab.chat._add (sTo, sBody, true, date);
			else
				tab.chat._add (sFrom, sBody, false, date);
				
			//Save history
			if (!h)
			    h = [];
			else
			if (h.length>49)
			    h = h.slice(h.length-49)

			h.push({from: sTo || sFrom,body:sBody,reply:sTo?true:false,date:date});
			dataSet.add('xmpp',['roster',sFrom,'history'],h,true);
		}
	}
	else{
		if (this._docked)
			this._undock();

		this._tab(sFrom,true);
	}
};
